@@ -5,7 +5,6 @@ create table t1 (  primary a int not null, b string(5) );
 alter table t1 add column c tinyint;
 desc table t1;
 
-create index i1 on t1 ( b );
 alter table t1 modify column b string(10);
 alter table t1 add column d string(10);
 alter table t1 add column e string(10);
@@ -15,7 +14,6 @@ alter table t1 drop column f, drop column g, drop column e;
 alter table t1 add column d string(5);
 
 desc table t1;
-desc index i1;
 
 insert into t1 values ( 0, 'xxx', (tinyint)1, 'XXX');
 insert into t1 values ( 1, 'yyy', (tinyint)2, 'XXX');
@@ -26,7 +24,6 @@ select * from t1;
 alter table t1 drop column b;
 desc table t1;
 
-
 create table t2 ( primary a int not null, b int );
 
 list table;
