@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2019 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -53,8 +53,6 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
 					      bool insertAtLastPage,
 					      CegoDataPointer& ritp)
 {
-    // get entry pointer
-
     int tabSetId = ioe.getTabSetId();
     Chain indexName = ioe.getName();
     Chain tabName = ioe.getTabName();
@@ -85,7 +83,7 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
     }
 
     // cout << "Write Locking " << rdp.getFileId() << " " << rdp.getPageId()  << endl;
-    _lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, rdp.getFileId(), rdp.getPageId(), CegoLockHandler::WRITE);
+    _lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, rdp.getPageId(), CegoLockHandler::WRITE);
     
     CegoAVLIndexEntry base;
     base.setPtr(p, len);
@@ -165,12 +163,8 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 					bool insertAtLastPage, 
 					bool allowWrite)
 {
-
-
-
-    // cout << "Write Locking " << ritp.getFileId() << " " << ritp.getPageId()  << endl;
     if ( _lockId == 0 )
-	_lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, ritp.getFileId(), ritp.getPageId(), CegoLockHandler::WRITE);
+	_lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, ritp.getPageId(), CegoLockHandler::WRITE);
           
     int tabSetId = ioe.getTabSetId();
     ListT<CegoField> schema = ioe.getSchema();
@@ -185,14 +179,16 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
     else
 	fixMode = CegoBufferPool::SYNC;
 
-    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ritp, p, len);
+    CegoBufferPage rootPage;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ritp, p, len, rootPage);
 
     CegoAVLIndexEntry base;
     base.setPtr(p, len);
 
     CegoDataPointer itp = base.getRightBranch();
 
-    CegoBufferPage pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+    CegoBufferPage pageIE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 
     CegoDataPointer ptp;
     
@@ -241,7 +237,8 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		// the key may be duplictate if an update operation occurs ( forced transaction )
 		// in this case we allow double entries ( cleanup by commit )
 
-		CegoBufferPage dataPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ie.getData(), p, len);
+		CegoBufferPage dataPage;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ie.getData(), p, len, dataPage);
 		
 		// skipping tid
 		unsigned long long dataTid; //  = *(unsigned long long*)p;
@@ -252,7 +249,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 
 		_pTM->releaseDataPtrUnlocked(dataPage, false);
 	    
-		if ( tid == dataTid && ( ts == DELETED || ts == OBSOLETE ))
+		if ( tid == dataTid && ( ts == CegoTupleState::DELETED || ts == CegoTupleState::OBSOLETE ))
 		{
 		    leftBranch = true;
 		    ptp = itp;
@@ -305,7 +302,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 	    {		
 		_pTM->releaseDataPtrUnlocked(pageIE, false);
 		_pTM->releaseDataPtrUnlocked(rootPage, false);
-		throw e;
+		throw Exception(EXLOC, Chain("Cannot insert index data"), e);
 	    }
 	    
 	    inserted = true;
@@ -344,7 +341,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 	{
 	    if ( pageIE.isFixed() )
 	    _pTM->releaseDataPtrUnlocked(pageIE, true);
-	    pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 
 	}
     }
@@ -372,8 +369,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
     ptp = ie.getParent();
 
     CegoBufferPage pagePE;
-
-    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 
     CegoAVLIndexEntry pe;       
     pe.setPtr(p, len);
@@ -402,7 +398,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		
 		if ( ptp.getOffset() )
 		{   
-		    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		    pe.setPtr(p, len);
 		}
 	    }
@@ -452,7 +448,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 
 		if ( ptp.getOffset() )
 		{
-		    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		    pe.setPtr(p, len);    
 		}
 	    }
@@ -513,7 +509,8 @@ void CegoAVLIndexManager::getSubTreeHeight(int tabSetId, CegoBufferPool::FixMode
     }
     else
     {
-	CegoBufferPage pageLE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len);
+	CegoBufferPage pageLE;
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len, pageLE);
 	CegoAVLIndexEntry li;       
 	li.setPtr(p, len);
 	leftHeight = li.getHeight();
@@ -529,7 +526,8 @@ void CegoAVLIndexManager::getSubTreeHeight(int tabSetId, CegoBufferPool::FixMode
     else
     {
     
-	CegoBufferPage pageRE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len);
+	CegoBufferPage pageRE;
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len, pageRE);
 	CegoAVLIndexEntry ri;       
 	ri.setPtr(p, len);
 	rightHeight = ri.getHeight();
@@ -576,7 +574,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     /////////////////////////////////////////
 
     CegoDataPointer ptp = rdp;
-    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode,  ptp, p, len);
+    CegoBufferPage rootPage;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode,  ptp, p, len, rootPage);
 
     CegoAVLIndexEntry ie;
     ie.setPtr(p, len);    
@@ -584,7 +583,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     CegoDataPointer  itp;
     itp = ie.getRightBranch();
         
-    CegoBufferPage pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);  
+    CegoBufferPage pageIE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);  
 
     bool entryFound = false;
 
@@ -628,7 +628,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		if ( itp != nil ) 
 		{
 		    CegoBufferPage bp;
-		    bp = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, bp);
 		    ie.setPtr(p, len);
 		    ptp = ie.getParent();
 		    if ( bp.isFixed() )
@@ -646,7 +646,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 	
 	    if (pageIE.isFixed() )
 		_pTM->releaseDataPtrUnlocked(pageIE, false);
-	    pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 	    // cout << "claimed ptr " << itp << endl;
 	}
     }
@@ -666,7 +666,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		
 		CegoAVLIndexEntry pie;
 		
-		CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		CegoBufferPage pagePE;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		pie.setPtr(p, len);	  	
 		
 		if ( pie.getLeftBranch() == itp )
@@ -698,7 +699,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 			    
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -741,7 +743,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 			    
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -768,10 +771,12 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		
 		CegoAVLIndexEntry pie, lie;
 		
-		CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		CegoBufferPage pagePE;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		pie.setPtr(p, len);	  	
 		
-		CegoBufferPage pageLE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len);
+		CegoBufferPage pageLE;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len, pageLE);
 		lie.setPtr(p,len);
 
 		lie.setParent(ptp);
@@ -805,7 +810,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 			    
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -846,7 +852,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -872,7 +879,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		
 		CegoAVLIndexEntry pie, rie;
 		
-		CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode,  ptp, p, len);
+		CegoBufferPage pagePE;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode,  ptp, p, len, pagePE);
 		pie.setPtr(p, len);	  	
 		
 		if ( pie.getLeftBranch() == itp )
@@ -901,7 +909,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 			    
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -943,7 +952,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    
 			    nptp = rebalanceNode(tabSetId, ptp, fixMode);
 			    
-			    CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+			    CegoBufferPage pageNE;
+			    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageNE);
 			    
 			    CegoAVLIndexEntry nie;
 			    nie.setPtr(p, len);
@@ -957,7 +967,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		}
 		_pTM->releaseDataPtrUnlocked(pagePE, true);	
 		
-		CegoBufferPage pageR = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len);
+		CegoBufferPage pageR;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len, pageR);
 		rie.setPtr(p,len);
 		rie.setParent(ptp);
 		
@@ -974,7 +985,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 
 		CegoAVLIndexEntry nie;
 		
-		CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ndp, p, len);
+		CegoBufferPage pageNE;
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ndp, p, len, pageNE);
 		nie.setPtr(p, len);
 		
 		if ( nie.getLeftBranch() == nil )
@@ -984,7 +996,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		    
 		    CegoAVLIndexEntry pie, lie;
 		    
-		    CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		    CegoBufferPage pagePE;
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		    pie.setPtr(p, len);
 		    
 		    if ( pie.getRightBranch() == itp )
@@ -1000,7 +1013,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		    
 		    CegoDataPointer ldp = ie.getLeftBranch();
 		    // cout << "LDP = " << ldp << endl;
-		    CegoBufferPage pageLE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len);
+		    CegoBufferPage pageLE;
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len, pageLE);
 		    
 		    lie.setPtr(p, len);
 		    
@@ -1024,7 +1038,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			char h = nie.getHeight();
 			CegoDataPointer nndp = rebalanceNode(tabSetId, ndp, fixMode);
 			
-			CegoBufferPage pageNN = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nndp, p, len);
+			CegoBufferPage pageNN;
+			_pTM->claimDataPtrUnlocked(tabSetId, fixMode, nndp, p, len, pageNN);
 			CegoAVLIndexEntry nnie;
 			nnie.setPtr(p, len);
 			
@@ -1060,13 +1075,13 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			    _pTM->releaseDataPtrUnlocked(pageNE, false);
 			}
 			
-			pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ndp, p, len);
+			_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ndp, p, len, pageNE);
 			nie.setPtr(p,len);	    		
 		    }
 		    
 		    CegoAVLIndexEntry fie;		    
 		    CegoDataPointer fdp = nie.getParent();
-		    pageFE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, fdp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, fdp, p, len, pageFE);
 		    fie.setPtr(p,len);
 		    
 		    if ( nie.getRightBranch() != nil )
@@ -1075,7 +1090,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			
 			CegoAVLIndexEntry zie;
 			
-			pageZE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, zdp, p, len);
+			_pTM->claimDataPtrUnlocked(tabSetId, fixMode, zdp, p, len, pageZE);
 			zie.setPtr(p,len);
 			
 			zie.setParent(nie.getParent());
@@ -1094,11 +1109,11 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		    bdp=ie.getLeftBranch();
 		    cdp=ie.getRightBranch();
 		    
-		    pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, adp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, adp, p, len, pageAE);
 		    aie.setPtr(p,len);
-		    pageBE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, bdp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, bdp, p, len, pageBE);
 		    bie.setPtr(p,len);
-		    pageCE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, cdp, p, len);
+		    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, cdp, p, len, pageCE);
 		    cie.setPtr(p,len);
 		    
 		    nie.setParent(adp);
@@ -1149,7 +1164,8 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			
 			nfdp = rebalanceNode(tabSetId, fdp, fixMode);
 			
-			CegoBufferPage pageRE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nfdp, p, len);
+			CegoBufferPage pageRE;
+			_pTM->claimDataPtrUnlocked(tabSetId, fixMode, nfdp, p, len, pageRE);
 			
 			CegoAVLIndexEntry nfie;
 			nfie.setPtr(p, len);
@@ -1214,26 +1230,22 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     catch ( Exception e )
     {
 	_pTM->releaseDataPtrUnlocked(rootPage, false);
-	throw e;
+	throw Exception(EXLOC, Chain("Cannot delete index data"), e);
     }
     
     if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, false);
     
-   
-    _pTM->releaseDataPtrUnlocked(rootPage, false);
-    
+    _pTM->releaseDataPtrUnlocked(rootPage, false);   
 }
 
 void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, CegoBufferPool::FixMode fixMode)
 {
-
-    // cout << "Propagate Decrease ..." << endl;
-
     char* p;
     int len;
 
-    CegoBufferPage pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+    CegoBufferPage pageIE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
     CegoAVLIndexEntry ie;    
     ie.setPtr(p, len);	  	
 
@@ -1247,7 +1259,8 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
     }
     CegoDataPointer ptp = ie.getParent();
     
-    CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+    CegoBufferPage pagePE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
     CegoAVLIndexEntry pie;    
     pie.setPtr(p, len);	  	
 
@@ -1294,7 +1307,7 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
 	    }
 	    if ( ptp.getOffset() )
 	    {   
-		pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+		_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 		pie.setPtr(p, len);
 	    }
 	}
@@ -1312,7 +1325,7 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
 	
 	if ( ptp.getOffset() )
 	{   
-	    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+	    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 	    pie.setPtr(p, len);
 	}
 	
@@ -1326,20 +1339,18 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
     if ( pageIE.isFixed() ) 	       
 	_pTM->releaseDataPtrUnlocked(pageIE, true);
     if ( pagePE.isFixed() )
-	_pTM->releaseDataPtrUnlocked(pagePE, true);
-    
+	_pTM->releaseDataPtrUnlocked(pagePE, true);    
 }
 
 CegoDataPointer CegoAVLIndexManager::rebalanceNode(int tabSetId, CegoDataPointer itp, CegoBufferPool::FixMode fixMode)
 {
-
-    // cout << "Rebalancing node " << itp << endl;
     char* p;
     int len;
 
     CegoDataPointer nil;
 
-    CegoBufferPage pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+    CegoBufferPage pageIE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
     CegoAVLIndexEntry ie;    
     ie.setPtr(p, len);	  	
 
@@ -1361,7 +1372,8 @@ CegoDataPointer CegoAVLIndexManager::rebalanceNode(int tabSetId, CegoDataPointer
 
 	// decide if LL or LR Rotation
 	CegoDataPointer ldp = ie.getLeftBranch();
-	CegoBufferPage pageLE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len);
+	CegoBufferPage pageLE;
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ldp, p, len, pageLE);
 	CegoAVLIndexEntry lie;    
 	lie.setPtr(p, len);	  	
 	
@@ -1385,7 +1397,8 @@ CegoDataPointer CegoAVLIndexManager::rebalanceNode(int tabSetId, CegoDataPointer
 	// decide if RR or RL Rotation
 
 	CegoDataPointer rdp = ie.getRightBranch();		
-	CegoBufferPage pageRE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len);
+	CegoBufferPage pageRE;
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len, pageRE);
 		
 	CegoAVLIndexEntry rie;
 	rie.setPtr(p, len);
@@ -1400,7 +1413,8 @@ CegoDataPointer CegoAVLIndexManager::rebalanceNode(int tabSetId, CegoDataPointer
 	else
 	{
 	    nptp = rotateRL(tabSetId, itp, fixMode);
-	    CegoBufferPage pageSE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len);
+	    CegoBufferPage pageSE;
+	    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, nptp, p, len, pageSE);
 	    
 	    CegoAVLIndexEntry rie;
 	    rie.setPtr(p, len);
@@ -1422,7 +1436,6 @@ CegoDataPointer CegoAVLIndexManager::rebalanceNode(int tabSetId, CegoDataPointer
 
 CegoAVLIndexManager::TupleOrder CegoAVLIndexManager::compIndexValue(const ListT<CegoField>& schema, char* p1, char* p2)
 {
-
     CegoField* pF = schema.First();
            	
     while ( pF )
@@ -1473,15 +1486,10 @@ CegoAVLIndexManager::TupleOrder CegoAVLIndexManager::compIndexValue(const ListT<
     }
    
     return EQU;
-
 }
 
-
 CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointer& ptp, CegoBufferPool::FixMode fixMode)
 {
-
-    // cout << "RotateRR" << ptp << endl;
-
     char *p;
     int len;
 
@@ -1498,14 +1506,14 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
     // a is parent of pe
     // c is left child of ie
 
-    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
     pe.setPtr(p, len);
 
     atp = pe.getParent();
           
     if ( atp.getOffset() )
     {
-	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len, pageAE);
 	ae.setPtr(p, len);
     }
 
@@ -1513,7 +1521,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
     
     if ( itp.getOffset() )
     {
-	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 	ie.setPtr(p, len);
     }
     else
@@ -1524,7 +1532,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
     ctp = ie.getLeftBranch();
     if ( ctp.getOffset() )
     {
-	pageCE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len, pageCE);
 	ce.setPtr(p, len);
     }
     
@@ -1582,7 +1590,6 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
 
 CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointer& ptp, CegoBufferPool::FixMode fixMode)
 {
-    // cout << "RotateRL at " << ptp << endl;
     char *p;
     int len;
     
@@ -1605,7 +1612,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 
     if ( ptp.getOffset() )
     {   
-	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 	pe.setPtr(p, len);
     }
     else
@@ -1617,7 +1624,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
         
     if ( atp.getOffset() )
     {
-	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len, pageAE);
 	ae.setPtr(p, len);
     }
 
@@ -1625,7 +1632,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 
     if ( itp.getOffset() )
     {
-	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 	ie.setPtr(p, len);
     }
     else
@@ -1636,21 +1643,21 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     ctp = ie.getLeftBranch();
     if ( ctp.getOffset() )
     {
-	pageCE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len, pageCE);
 	ce.setPtr(p, len);
     }
     
     ftp = ce.getRightBranch();
     if ( ftp.getOffset() )
     {
-	pageFE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ftp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ftp, p, len, pageFE);
 	fe.setPtr(p, len);
     }
         
     etp = ce.getLeftBranch();
     if ( etp.getOffset() )
     {
-	pageEE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, etp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, etp, p, len, pageEE);
 	ee.setPtr(p, len);
     }
     
@@ -1728,9 +1735,6 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 
 CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointer& ptp, CegoBufferPool::FixMode fixMode)
 {
-
-    // cout << "RotateLL at " << ptp << endl;
-
     char *p;
     int len;
     
@@ -1748,7 +1752,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
 
     if ( ptp.getOffset() )
     {   
-	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 	pe.setPtr(p, len);
     }
     else
@@ -1760,7 +1764,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     
     if ( atp.getOffset() )
     {	
-	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len, pageAE);
 	ae.setPtr(p, len);
     }
 
@@ -1768,7 +1772,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
 
     if ( itp.getOffset() )
     {
-	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 	ie.setPtr(p, len);
     }
     else
@@ -1780,7 +1784,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     
     if ( dtp.getOffset() )
     {
-	pageDE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len, pageDE);
 	de.setPtr(p, len);
     }
         
@@ -1834,9 +1838,6 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
 
 CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointer& ptp, CegoBufferPool::FixMode fixMode)
 {
-
-    // cout << "RotateLR at " << ptp  << endl;
-
     char *p;
     int len;
         
@@ -1859,7 +1860,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     
     if ( ptp.getOffset() )
     {   
-	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len, pagePE);
 	pe.setPtr(p, len);
     }
     else
@@ -1871,7 +1872,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     
     if ( atp.getOffset() )
     {
-	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len, pageAE);
 	ae.setPtr(p, len);
     }
 
@@ -1879,7 +1880,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     
     if ( itp.getOffset() )
     {
-	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len, pageIE);
 	ie.setPtr(p, len);
     }
     else
@@ -1890,7 +1891,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     dtp = ie.getRightBranch();
     if ( dtp.getOffset() )
     {	
-	pageDE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len, pageDE);
 	de.setPtr(p, len);
     }
     else
@@ -1901,14 +1902,14 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     ctp = de.getLeftBranch();
     if ( ctp.getOffset() )
     {
-	pageCE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len, pageCE);
 	ce.setPtr(p, len);
     }
 
     etp = de.getRightBranch();
     if ( etp.getOffset() )
     {
-	pageEE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, etp, p, len);
+	_pTM->claimDataPtrUnlocked(tabSetId, fixMode, etp, p, len, pageEE);
 	ee.setPtr(p, len);
     }
     
@@ -1980,9 +1981,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
 }
 
 char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoObject::ObjectType idxType)
-{    
-
-
+{
     CegoTableObject oe;
     _pTM->getObject(tabSetId, indexName, idxType, oe);
    
@@ -2010,7 +2009,8 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
     /////////////////////////////////////////
 
     CegoDataPointer ptp = rdp;
-    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, ptp, p, len);
+    CegoBufferPage rootPage;
+    _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, ptp, p, len, rootPage);
 
     CegoAVLIndexEntry ie;
     ie.setPtr(p, len);    
@@ -2027,7 +2027,6 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
 
 char CegoAVLIndexManager::recursiveIndexNodeCheck(int tabSetId, const CegoDataPointer& dp)
 {
-
     CegoDataPointer nil;
 
     if ( dp == nil )
@@ -2040,7 +2039,8 @@ char CegoAVLIndexManager::recursiveIndexNodeCheck(int tabSetId, const CegoDataPo
     _pLogger->log(_modId, Logger::DEBUG, Chain("Claiming datapointer ") +  dp.toChain());
 #endif
 
-    CegoBufferPage bp = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len); 
+    CegoBufferPage bp;
+    _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len, bp); 
 
     CegoAVLIndexEntry ie;
     ie.setPtr(p, len);    
@@ -2085,8 +2085,7 @@ char CegoAVLIndexManager::recursiveIndexNodeCheck(int tabSetId, const CegoDataPo
     }
 
     return nodeHeight;
- }
-
+}
 
 CegoDataPointer CegoAVLIndexManager::searchDataPointer(int tabSetId, const CegoDataPointer& ddp, const CegoDataPointer& rdp, CegoBufferPool::FixMode fixMode)
 {
@@ -2100,7 +2099,8 @@ CegoDataPointer CegoAVLIndexManager::searchDataPointer(int tabSetId, const CegoD
     char* p;
     int len;
          
-    CegoBufferPage pageRE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len);  
+    CegoBufferPage pageRE;
+    _pTM->claimDataPtrUnlocked(tabSetId, fixMode, rdp, p, len, pageRE);  
 
     CegoAVLIndexEntry re;
     re.setPtr(p, len);    
