@@ -237,6 +237,7 @@ public:
     void storeLogNumAttr();
     void storeSortAreaAttr();
     void storeTSRootAttr();
+    void storeBranchAttr();
 
     void storePit();
 
@@ -273,6 +274,7 @@ private:
     int _logSize;
     int _logNum;
     unsigned long long _sortAreaSize;
+    int _branchId;
     Chain _pit;
 
     bool _rawMode;
