@@ -962,24 +962,25 @@ void helpTopic(const Chain& topic)
 	cout << "define tableset <tableset> <attr1=val1> <attr2=val2> ..." << endl;
 	cout << "--------------------------------------------------------" << endl;
 	cout << endl;
-	cout << "   Define a new tableset. The following attributes must be specified:"<< endl;
+	cout << "   Define a new tableset. The following attributes can be specified:"<< endl;
 	cout << endl;
 	cout << "   tsroot          - Root directory of the database system and control files." << endl;
 	cout << "   syssize         - Size of the system space in number of pages." << endl;
 	cout << "   tmpsize         - Size of the tempory space in number of pages." << endl;
 	cout << "   appsize         - Size of the application space in number of pages." << endl;
-	cout << "   logname         - Prefix of the configured redolog files." << endl;
 	cout << "   logsize         - Size of a redolog file in bytes." << endl;
 	cout << "   lognum          - Number of configured redolog files." << endl;
+	cout << "   sortareasize    - Size of the sort area in byte." << endl;
+	cout << "   branchid        - Number of the backup branch." << endl;
 	cout << "   primary         - Name of the primary host." << endl;
 	cout << "   secondary       - Name of the secondary host." << endl;
 	cout << endl;
-	cout << "   For a defintion statement the attributes syssize, tmpsize logsize and logname attributes are specified as integer values." << endl;
-	cout << "   The attributes logfilename, meiator, primary and secondary  are specified as string values." << endl;
+	cout << "   For a defintion statement the attributes syssize, tmpsize, appsize, logsize, lognum, sortareasize and branchid attributes are specified as integer values." << endl;
+	cout << "   The attributes tsroot, primary and secondary are specified as string values." << endl;
 	cout << endl;
 	cout << "   Sample:" << endl;
 	cout << endl;
-	cout << "      define tableset ts1 tsroot='/cego/ts1' syssize=100 tmpsize=100 appsize=3000 logname='/cego/ts1/redolog'" << endl; 
+	cout << "      define tableset ts1 tsroot='/cego/ts1' syssize=100 tmpsize=100 appsize=3000 " << endl; 
 	cout << "             logsize=100000 lognum=3 primary='geek' secondary='sam';" << endl;
 	cout << endl;
     }
