@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -38,9 +38,9 @@ CegoAttrDesc::CegoAttrDesc(const Chain& attrName)
     _isValid = false;
 }
 
-CegoAttrDesc::CegoAttrDesc(Element *pADE)
+CegoAttrDesc::CegoAttrDesc(char *buf)
 {
-    fromElement(pADE);
+    decode(buf);
     _isValid = false;
 }
 
@@ -98,14 +98,14 @@ CegoAttrDesc& CegoAttrDesc::operator = ( const CegoAttrDesc& ad )
     return (*this);
 }
 
-bool CegoAttrDesc::operator == ( const CegoAttrDesc& ad)
+bool CegoAttrDesc::operator == ( const CegoAttrDesc& ad) const
 {    
     if (_tableName == ad._tableName && _attrName == ad._attrName)
 	return true;
     return false;
 }
 
-bool CegoAttrDesc::operator != ( const CegoAttrDesc& ad)
+bool CegoAttrDesc::operator != ( const CegoAttrDesc& ad) const
 {    
     if (_tableName == ad._tableName && _attrName == ad._attrName)
 	return false;
@@ -137,20 +137,6 @@ Chain CegoAttrDesc::dbFormat(CegoDatabaseFormater* pForm) const
     return pForm->formatAttr(_tableName, _attrName);
 }
 
-Element* CegoAttrDesc::toElement() const
-{
-    Element *pAttrDescElement = new Element( XML_ATTRDESC_ELEMENT );
-    pAttrDescElement->setAttribute( XML_TABLENAME_ATTR, _tableName );
-    pAttrDescElement->setAttribute( XML_ATTRNAME_ATTR, _attrName );
-    return pAttrDescElement;
-}
-
-void CegoAttrDesc::fromElement(Element *pAttrDescElement)
-{
-    _tableName = pAttrDescElement->getAttributeValue(XML_TABLENAME_ATTR);
-    _attrName = pAttrDescElement->getAttributeValue(XML_ATTRNAME_ATTR);
-}
-
 void CegoAttrDesc::encode(char *buf)
 {   
     char* pE = (char*)buf;
@@ -191,7 +177,6 @@ void CegoAttrDesc::encode(char *buf)
 void CegoAttrDesc::decode(char *buf)
 {
     char* pE = (char*)buf;
-
     char len;
     
     memcpy( &len, pE, sizeof(char));
@@ -221,7 +206,7 @@ int CegoAttrDesc::getEncodingLength() const
     
     len += sizeof(char);
     len += _tableName.length();
-
+    
     len += sizeof(char);
     len += _attrName.length();
 
@@ -241,10 +226,11 @@ int CegoAttrDesc::evalReferences(CegoContentObject *pCO, const ListT<CegoField>&
 	    pSubCO = pCO->getSubCOList().Next();
 	}
     }
-    else if ( pCO->getType() == CegoObject::VIEW || pCO->getType() == CegoObject::TABLE || pCO->getType() == CegoObject::SYSTEM )
+    else if ( pCO->getType() == CegoObject::VIEW || pCO->getType() == CegoObject::TABLE ||  pCO->getType() == CegoObject::ALIAS || pCO->getType() == CegoObject::SYSTEM )
     {
 	refCount += evalTableReferences(pCO, fl);
     }
+    
     return refCount;
 }
 
@@ -252,9 +238,13 @@ int CegoAttrDesc::evalTableReferences(CegoContentObject *pCO, const ListT<CegoFi
 {
     int refCount = 0;
     
-    if ( _tableName == (Chain)pCO->getTabName() || ( _tableName == (Chain)pCO->getTabAlias() && _tableName != Chain()) )
+    if ( ( _tableName == (Chain)pCO->getTabName() && pCO->getType() != CegoObject::ALIAS )
+	|| ( _tableName == (Chain)pCO->getTabAlias() && _tableName != Chain()) )
     {
-	_tableName = pCO->getName();
+	if ( pCO->getType() == CegoObject::ALIAS )
+	    _tableName = pCO->getTabAlias();
+	else
+	    _tableName = pCO->getName();
 	refCount++;
     }
     else if ( _tableName == Chain() ) // || pCO->getType() == CegoObject::JOIN )
@@ -264,11 +254,8 @@ int CegoAttrDesc::evalTableReferences(CegoContentObject *pCO, const ListT<CegoFi
 	{
 	    if (_attrName == (Chain)pF->getAttrName())
 	    {
-		// _tableName = pF->getTableName();
-		// cout << "Set table to " << pF->getTableAlias() << endl;
 		_tableName = pF->getTableAlias();
 		refCount++;
-		// _tableName = pCO->getName();
 	    }	
 	    pF = fl.Next();	    
 	}
