@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -25,6 +25,7 @@
 #include <lfcbase/ListT.h>
 
 // CEGO INCLUDES
+#include "CegoDefs.h"
 #include "CegoObject.h"
 #include "CegoField.h"
 #include "CegoAttrCond.h"
@@ -51,7 +52,17 @@ public:
     
 private:
 
-    CegoComparison compValue(const CegoBTreeValue& val);
+    enum Direction { UP, DOWN };
+    
+    void prepareComp();
+
+    Direction traceLow(const CegoBTreeValue& val);
+    bool inRange(const CegoBTreeValue& val);
+    
+    bool fullMatch(const CegoBTreeValue& val);
+
+    // optimized merged version 
+    void checkRangeAndMatch(const CegoBTreeValue& val, bool& inRange, bool& doMatch);
     
     CegoAttrCond* _pAttrCond;
 
@@ -61,12 +72,9 @@ private:
     Chain _btreeName;
     CegoObject::ObjectType _type;
 
-    int _rootFileId;
-    int _rootPageId;
-
-    int _curFileId;
-    int _curPageId;
-    int _curPos;
+    PageIdType _rootPageId;
+    
+    PageIdType _curPageId;
     CegoBTreeNode _curLeaf;
     CegoBufferPage _curPage;
     
@@ -79,13 +87,26 @@ private:
 
     CegoQueryHelper _qh;
 
+    bool _btwCompAdjusted;
     bool _cursorCached;
     bool _eoc;
 
+    struct CompInfo
+    {
+	CegoAttrComp* pAttrComp;
+	CegoDataType compType;
+	int compLen;
+	int compPos;
+    } _compInfo[TABMNG_MAXBTREECOMP];
+    
+    int _numComp;
+    
     unsigned int _lockId;
     
     ListT<CegoField> _btreeSchema;
     int _keyLen;
     StackT<CegoBufferPage> _parentPageStack;
+
+    unsigned long _modId;
 };
 #endif
