@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2010-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -31,42 +31,34 @@
 #include "CegoClob.h"
 
 class CegoNet {
-
- public:
+    
+public:
 
     CegoNet(CegoDbHandler::ProtocolType protType, const Chain& logFile, const Chain& progName, const Chain& logMode );
     ~CegoNet();
     
-    void connect(const Chain& serverName, int port, const Chain& tableSet, const Chain& user, const Chain& pwd);
-    long doQuery(const Chain& query);
-
+    void connect(const Chain& serverName, int port, const Chain& tableSet, const Chain& user, const Chain& pwd, int maxSendLen = NETMNG_MAXSENDLEN);
+    unsigned long long doQuery(const Chain& query);
     void putBlob(CegoBlob& blob);
     void putClob(CegoClob& clob);
-
     void getBlob(CegoBlob& blob);
     void getClob(CegoClob& clob);
-
     void deleteClob(CegoClob& clob);
     void deleteBlob(CegoBlob& blob);
-    
     bool fetchData(const ListT<CegoField>& schema, ListT<CegoFieldValue>& fvl);
-
-    
     bool isFetchable();
     void abortQuery();
     void resetQuery();
     bool wasReset();
-
     unsigned long long getTid() const;
+    const Chain& getDbName() const;
+    const Chain& getDbVersion() const;
     void reqAbortQuery(unsigned long long tid);
-    
     const Chain& getMsg();
     long getAffected();
     void getSchema(ListT<CegoField>& schema);
     void getFormat(Chain& format);
-    
     void getProcResult(ListT<CegoProcVar>& outParamList, CegoFieldValue& retValue);
-
     void disconnect();
 
 private:
@@ -75,11 +67,9 @@ private:
     Chain _progName;
     Chain _logFile;
     Chain _logMode;
-
     NetHandler* _pN;
     CegoDbHandler* _pSH;
     CegoModule* _pModule;
-
     bool _isFetchable;
 };
 #endif
