@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,6 +22,7 @@
 
 // LFC INCLUDES
 #include <lfcbase/Chain.h>
+
 // CEGO INCLUDES
 #include "CegoProcStmt.h"
 #include "CegoExpr.h"
@@ -32,18 +33,17 @@ class CegoProcCursorCreateStmt : public CegoProcStmt {
 public:
     
     CegoProcCursorCreateStmt(const Chain& name, CegoSelect *pSelect, CegoProcBlock *pBlock);
-    
     ~CegoProcCursorCreateStmt();
 
     CegoException execute();
-    
-    Chain toChain(const Chain& indent) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
     
     Chain _name;
-    CegoSelect *_pSelect;
-    
+    CegoSelect *_pSelect;    
 };
 #endif
