@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoProcObject.cc 
-// ------------------
+// -----------------
 // Cego procedure object implementation
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -57,7 +57,7 @@ int CegoProcObject::getEntrySize() const
     return entrySize;
 }
 
-void CegoProcObject::encode(char *buf)
+void CegoProcObject::encode(char *buf) const
 {
     char* bufPtr = buf;
     int entrySize = getEntrySize();
@@ -103,33 +103,6 @@ Chain CegoProcObject::toChain() const
     return s;
 }
 
-Element* CegoProcObject::getElement() const
-{
-    Element* pRoot = new Element(XML_OBJ_ELEMENT);
-
-    pRoot->setAttribute(XML_TSID_ATTR, Chain(getTabSetId()));
-    pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_PROCOBJ_VALUE);	
-    pRoot->setAttribute(XML_OBJNAME_ATTR, getName());
-    pRoot->setAttribute(XML_PROCTEXT_ATTR, _procText);
-
-    return pRoot;
-}
-
-void CegoProcObject::putElement(Element* pElement)
-{
-    Element *pRoot = pElement;
-    
-    if ( pRoot )
-    {
-	int tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
-	setTabSetId(tabSetId);
-	Chain objName = pRoot->getAttributeValue(XML_OBJNAME_ATTR);
-	setName(objName);
-	setType(CegoObject::PROCEDURE);
-	_procText = pRoot->getAttributeValue(XML_PROCTEXT_ATTR);
-    }
-}
-
 Chain CegoProcObject::getFormatted() const
 {
     Chain s;
@@ -151,7 +124,6 @@ Chain CegoProcObject::getFormatted() const
     s += "| Procedure Text        " +  fill(" ", maxFillLen-23) + "|\n";
     s += "+" + fill("-", maxFillLen) + "+\n";
 	
-
     Tokenizer pt(_procText, "\n");
     
     while ( pt.nextToken(line) )
@@ -165,8 +137,7 @@ Chain CegoProcObject::getFormatted() const
 
 Chain CegoProcObject::fill(const Chain& s, int num) const
 {
-    
-    Chain fs = Chain("");
+    Chain fs = Chain();
     while (num > 0)
     {
 	fs = fs + s;
