@@ -8,7 +8,7 @@
 //              
 // Design and Implementation by Bjoern Lemke
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke   
+// (C)opyright 2000-2025 Bjoern Lemke   
 //
 // INTERFACE MODULE
 //
@@ -24,29 +24,30 @@
 // cego includes
 #include "CegoProcBlock.h"
 #include "CegoProcStmt.h"
-#include "CegoProcCond.h"
+#include "CegoCondition.h"
 
 class CegoProcWhileStmt : public CegoProcStmt {
     
 public:
-
-    CegoProcWhileStmt(CegoProcCond  *pCond,
+    
+    CegoProcWhileStmt(CegoTableManager *pTabMng,
+		      CegoPredicate  *pPred,
 		      CegoProcBlock *pBlock,
 		      CegoProcBlock *pParentBlock);
-
     ~CegoProcWhileStmt();
 
     void setBlock(CegoProcBlock* pBlock);
-    
     CegoException execute();
-  
-    Chain toChain(const Chain& indent = Chain("")) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
 
-    CegoProcCond *_pCond;
+    CegoTableManager *_pTabMng;
+    CegoPredicate *_pPred;
     CegoProcBlock* _pBlock;
+    CegoProcBlock* _pParentBlock;
 };
-
 #endif
