@@ -6,7 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -19,6 +19,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 
 // cego includes
+#include "CegoDefs.h"
 #include "CegoProcedure.h"
 #include "CegoTypeConverter.h"
 #include "CegoProcException.h"
@@ -33,23 +34,30 @@ CegoProcedure::CegoProcedure(const Chain& procName, CegoProcBlock *pBlock)
     _procName = procName;
     _pBlock = pBlock;    
     _procType = PROCEDURE;
+    _returnType = NULL_TYPE;
+    _returnTypeLen = 0;
+    _returnTypeDim = 0;
     _pMasterBlock = 0;
+    _cacheEnabled = false;
 }
  
-CegoProcedure::CegoProcedure(const Chain& procName, CegoProcBlock *pBlock, CegoDataType returnType, int returnTypeLen)
+CegoProcedure::CegoProcedure(const Chain& procName, CegoProcBlock *pBlock, CegoDataType returnType, int returnTypeLen, int returnTypeDim)
 {
     _procName = procName;
     _pBlock = pBlock;
     _returnType = returnType;
     _returnTypeLen = returnTypeLen;
+    _returnTypeDim = returnTypeDim;
     _procType = FUNCTION;
     _pMasterBlock = 0;
+    _cacheEnabled = false;
 }
 
 CegoProcedure::~CegoProcedure()
 {
+    cleanUp();
     if ( _pBlock )
-	delete _pBlock;
+       delete _pBlock;
 }
 
 void CegoProcedure::setMasterBlock(CegoProcBlock *pBlock)
@@ -57,6 +65,11 @@ void CegoProcedure::setMasterBlock(CegoProcBlock *pBlock)
     _pMasterBlock = pBlock;
 }
 
+CegoProcBlock* CegoProcedure::getMasterBlock() const
+{
+    return _pMasterBlock;
+}
+
 const Chain& CegoProcedure::getName() const
 {
     return _procName;
@@ -84,48 +97,38 @@ void CegoProcedure::getArgList(ListT<CegoProcVar>& argList)
     }    
 }
 
-void CegoProcedure::execute(const ListT<CegoExpr*>& expList)
-{
-    CegoExpr **pExpr = expList.First();
+void CegoProcedure::execute(const ListT<CegoFieldValue>& fvl)
+{    
+    CegoFieldValue *pValue = fvl.First();
     CegoProcVar *pVar = _pBlock->getVarList().First();
 
+    // for proc cache
+    ListT<CegoFieldValue> infvl;
+
     // we just treat the in out param values at the beginning of the param list in the block
     if ( pVar )
 	if ( pVar->getVarType() == CegoProcVar::BLOCKVAR )
 	    pVar = 0;
     
-    int pos=1;
-    while ( pExpr && pVar)
+    while ( pValue && pVar)
     {
 	if ( pVar->getVarType() == CegoProcVar::INVAR )
 	{
-	    CegoFieldValue fv = (*pExpr)->evalFieldValue();
-
-	    if ( fv.getType() == NULL_TYPE )
+	    if ( pValue->getType() == VARCHAR_TYPE )
 	    {
-		// nothing to to
-	    }
-	    else if ( pVar->getType() != fv.getType() )		
-	    {
-		if ( fv.castTo(pVar->getType(), pVar->getLength()) == false )
-		{
-		    throw Exception(EXLOC, Chain("Mismatched datatype <") 
-				    + CEGO_TYPE_MAP[(int)fv.getType()] 
-				    + Chain("> in value list for argument ") + Chain(pos) 
-				    + " ( expected " 
-				    + CEGO_TYPE_MAP[(int)pVar->getType()] + " )");
-		}
-	    }
-	    else if ( fv.getType() == VARCHAR_TYPE )
-	    {
-		if ( pVar->getLength() < fv.getLength() - 1 )
+		if ( pVar->getLength() < pValue->getLength() - 1 )
 		    throw Exception(EXLOC, Chain("Parameter length for ") + pVar->getName() + Chain(" exceeded ( max len is ") + Chain(pVar->getLength()) + Chain(")"));
 	    }
+	    
+	    pVar->setValue(*pValue);
 
-	    pVar->setValue(fv);
+	    if ( _cacheEnabled && _pBlock->isStatic() )
+	    {
+		infvl.Insert(*pValue);
+	    }
 	}
-	pos++;
-	pExpr = expList.Next();
+	
+	pValue = fvl.Next();
 	pVar = _pBlock->getVarList().Next();
 	
 	// we just treat the in out param values at the beginning of the param list in the block
@@ -134,45 +137,63 @@ void CegoProcedure::execute(const ListT<CegoExpr*>& expList)
 		pVar = 0;
     }
 
-    if ( pVar || pExpr )
-    {
+    if ( pVar || pValue )
+    {	
 	Chain msg = Chain("Mismatched parameter count for procedure ");
 	throw Exception(EXLOC, msg);
     }
 
+    if ( _cacheEnabled && _pBlock->isStatic() )
+    {
+	CegoFieldValue procResult;
+        if ( getCacheValue(infvl, procResult) )
+	{
+	    // cout << "Found result .." << procResult << endl;
+	    _pBlock->setRetVal(procResult);
+	    return;
+	}
+    }
+    	
     CegoException excep = _pBlock->execute();
-
+    
+    if ( _cacheEnabled && _pBlock->isStatic() )
+    {
+	addCacheValue(infvl, _pBlock->getRetVal());
+    }
+    
     if ( excep != NONE_EXCEP && excep != RETURN_EXCEP )
     {
 	Chain msg = Chain("Procedure exception : ") + _pBlock->getExceptionMsg();
 	throw Exception(EXLOC, msg);
     }
 
-    pExpr = expList.First();
+    pValue = fvl.First();
     pVar = _pBlock->getVarList().First(); 
     _outParamList.Empty();
 
-    while ( pExpr && pVar)
+    while ( pValue && pVar)
     {
 	if ( pVar->getVarType() == CegoProcVar::OUTVAR )
 	{
-	    // CegoFieldValue fv = (*pExpr)->evalExpr();
+	    Chain var = pValue->valAsChain();
+	    
 	    if ( _pMasterBlock )
 	    {
-		Chain var;
-		(*pExpr)->checkVar(var);
 		_pMasterBlock->setValue(var, pVar->getValue());
 	    }
-	    Chain var;
-	    (*pExpr)->checkVar(var);
 	    
 	    CegoFieldValue fv = pVar->getValue();
-	    _outParamList.Insert(CegoProcVar(var, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv));
+	    _outParamList.Insert(CegoProcVar(var, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv.getDim(), fv));
 	}
-	pExpr = expList.Next();
-	pVar = _pBlock->getVarList().Next();
-		
-    }    
+	pValue = fvl.Next();
+	pVar = _pBlock->getVarList().Next();	
+    }
+}
+
+void CegoProcedure::cleanUp()
+{
+    _pMasterBlock = 0;
+    _pBlock->cleanUp();
 }
 
 CegoDataType CegoProcedure::getReturnType() const
@@ -185,22 +206,21 @@ int CegoProcedure::getReturnTypeLen() const
     return _returnTypeLen;
 }
 
+int CegoProcedure::getReturnTypeDim() const
+{
+    return _returnTypeDim;
+}
+
 CegoFieldValue CegoProcedure::getRetVal() const
 {
-    
-    if ( _pBlock->getRetVal().getType() != _returnType )
+    // still we have to cast for type or dim adjustments
+    if ( _pBlock->getRetVal().castTo(_returnType, _returnTypeDim) == false )
     {
-		
-	if ( _pBlock->getRetVal().castTo(_returnType, _returnTypeLen) == false )
-	{
-	    CegoTypeConverter tc;
-	    Chain msg = Chain("Cannot cast to type ")  + tc.getTypeString(_returnType);;
-	    throw Exception(EXLOC, msg);	
-	}	
-    }
+	Chain msg = Chain("Cannot cast to type ")  + CegoTypeConverter::getTypeString(_returnType);
+	throw Exception(EXLOC, msg);	
+    }	
 
     return _pBlock->getRetVal();
-
 }
 
 CegoProcedure::ProcType CegoProcedure::getProcType() const
@@ -216,20 +236,20 @@ CegoProcedure& CegoProcedure::operator = ( const CegoProcedure& p)
     _procType = p._procType;
     _returnType = p._returnType;
     _returnTypeLen = p._returnTypeLen;
+    _returnTypeDim = p._returnTypeDim;
+    _cacheEnabled = p._cacheEnabled;
     return (*this);
 }
 
 bool CegoProcedure::operator == ( const CegoProcedure& p) const
 {
-
     if ( _procName == p._procName )
 	return true;
     return false;
 }
 
-Chain CegoProcedure::toChain() const
+Chain CegoProcedure::toChain(int defTabSetId) const
 {
-
     Chain s;
     s = Chain("procedure ") + _procName + Chain("(");
     CegoProcVar *pVar = _pBlock->getVarList().First();
@@ -286,12 +306,7 @@ Chain CegoProcedure::toChain() const
 	    break;
 	case DECIMAL_TYPE:
 	    s += Chain(" decimal(");
-	    s += Chain(_returnTypeLen);
-	    s += Chain(")");
-	    break;
-	case FIXED_TYPE:
-	    s += Chain(" fixed(");
-	    s += Chain(_returnTypeLen);
+	    s += Chain(_returnTypeLen) + Chain(",") + Chain(_returnTypeDim);
 	    s += Chain(")");
 	    break;
 	case SMALLINT_TYPE:
@@ -307,15 +322,17 @@ Chain CegoProcedure::toChain() const
 	    throw Exception(EXLOC, Chain("Blob not supported as return type"));
 	case CLOB_TYPE:
 	    throw Exception(EXLOC, Chain("Clob not supported as return type"));
+	case PAGEID_TYPE:
+	    throw Exception(EXLOC, Chain("Return type not supported"));
 	}
     }
 
     s += Chain("\n");
     s += Chain("begin\n");
 
-    Chain indent = Chain("   ");
+    Chain indent = Chain(DEFAULTINDENT);
 
-    s += _pBlock->toChain(indent);
+    s += _pBlock->toChain(defTabSetId, indent);
     
     s += Chain("end");
     return s;
@@ -323,5 +340,91 @@ Chain CegoProcedure::toChain() const
 
 Chain CegoProcedure::dbFormat(CegoDatabaseFormater *pForm) const
 {
-    return pForm->formatProcedure(_procName, _procType, _returnType, _returnTypeLen, _pBlock);
+    return pForm->formatProcedure(_procName, _procType, _returnType, _returnTypeLen, _returnTypeDim, _pBlock);
+}
+
+bool CegoProcedure::getCacheValue(const ListT<CegoFieldValue>& fvl, CegoFieldValue& fv)
+{
+    ProcCacheValue *pPCV = _cacheSet.Find( ProcCacheValue(fvl));
+    if ( pPCV )
+    {
+	fv = pPCV->getRetVal();
+	return true;
+    }
+	
+    return false;
+}
+
+void CegoProcedure::addCacheValue(const ListT<CegoFieldValue>& fvl, const CegoFieldValue& retVal)
+{
+    if ( _cacheSet.Size() > PROCQUERYCACHE_MAXNUM )
+    {
+	ProcCacheValue *pPCV = _cacheSet.First();
+	ProcCacheValue *pLast = 0;
+	while ( pPCV )
+	{
+	    pLast = pPCV;
+	    pPCV = _cacheSet.Next();
+	}
+	if ( pLast )
+	{
+	    _cacheSet.Remove(*pLast);
+	}
+    }
+    _cacheSet.Insert( ProcCacheValue(fvl, retVal) );
+}	
+
+void CegoProcedure::enableProcCache(bool isEnabled)
+{
+    _cacheEnabled = isEnabled;
+}
+    
+CegoProcedure::ProcCacheValue::ProcCacheValue(const ListT<CegoFieldValue>& fvl, const CegoFieldValue& retVal)
+{
+    CegoFieldValue *pFV = fvl.First();
+    while ( pFV )
+    {
+	_paramHash += pFV->valAsChain();
+	pFV = fvl.Next();
+	if ( pFV )
+	    _paramHash += Chain("#");
+    }
+    _retVal = retVal;
+}
+
+CegoProcedure::ProcCacheValue::ProcCacheValue(const ListT<CegoFieldValue>& fvl)
+{
+    CegoFieldValue *pFV = fvl.First();
+    while ( pFV )
+    {
+	_paramHash += pFV->valAsChain();
+	pFV = fvl.Next();
+	if ( pFV )
+	    _paramHash += Chain("#");
+    }
+}
+
+CegoProcedure::ProcCacheValue::ProcCacheValue()
+{
+}
+
+CegoProcedure::ProcCacheValue::~ProcCacheValue()
+{
+}
+
+CegoProcedure::ProcCacheValue& CegoProcedure::ProcCacheValue::operator = (const CegoProcedure::ProcCacheValue& pcv)
+{
+    _paramHash = pcv._paramHash;
+    _retVal = pcv._retVal;
+    return (*this);
+}	
+
+bool CegoProcedure::ProcCacheValue::operator == (const CegoProcedure::ProcCacheValue& pcv) const
+{
+    return _paramHash == pcv._paramHash;
+}
+
+CegoFieldValue& CegoProcedure::ProcCacheValue::getRetVal()
+{
+    return _retVal;
 }
