@@ -57,7 +57,7 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, l
     }
     
     int tabSetId = _pDBMng->getTabSetId(tableSet);
-
+    
     _pDBMng->setTableSetRunState(tableSet, XML_RECOVERY_VALUE);
     _pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::ON ); 
 
@@ -93,22 +93,16 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, l
 	if ( pAH ) 
             pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Stopping log ...\n")); 
 	_pDBMng->stopLog(tabSetId);	
-
-	if ( pAH ) 
-            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering datafiles ...\n")); 
-	_pGTM->regDataFiles(tableSet);
-
-	if ( pAH ) 
-            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering objects ...\n")); 
-	_pGTM->registerObjects(tableSet);
-	
-	unsigned long long cplsn;
 	
 	Chain tsTicketName = _pDBMng->getTSTicket(tableSet);
-	
+
+	bool needFileRecovery=false;
 	File tsTicket(tsTicketName);
 	if ( tsTicket.exists() ) 
 	{
+
+	    needFileRecovery=true;
+	    
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Backup tableset ticket detected, datafile file recovery required ..."));
 	    tsTicket.open(File::READ);
 	    
@@ -125,55 +119,70 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, l
 	    Element *pRoot = pDoc->getRootElement();
 	    pDoc->setRootElement(0);
 	    delete pDoc;
-
+	    
+	    //  we have to check branchId consistency
+	    
 	    Chain buId = pRoot->getAttributeValue(XML_BRANCHID_ATTR);
 	    Chain currentId = _pDBMng->getTableSetBackupBranch(tableSet);
-
+	    
 	    if ( buId == currentId )
-		_pDBMng->log(_modId, Logger::NOTICE, Chain("Backup branch for ticket file verified"));
+		_pDBMng->log(_modId, Logger::NOTICE, Chain("Backup branch id ") + buId + Chain(" for ticket file verified"));
 	    else
 	    {
 		delete pRoot;
-
+			
 		Chain msg = Chain("Backup branch mismatch : <") + buId + Chain("> != <") + currentId + Chain(">");
 		
 		_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );
 		_pDBMng->log(_modId, Logger::LOGERR, msg);
-
+		
 		throw Exception(EXLOC, msg);
-	    }	
+	    }
 	    
 	    _pDBMng->setTableSetInfo(tableSet, pRoot);
 	    
-	    cplsn = _pDBMng->getCommittedLSN(tableSet);
-
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Committed lsn = ") + Chain(cplsn));
+	    tsTicket.close();
+    
+#ifdef CGDEBUG	
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Removing backup ticket ") + tsTicket.getFileName() + Chain(" ..."));
+#endif
 	    
-	    if ( pAH ) 
+	    tsTicket.remove();
+
+	    // after ticket restore, we have to set tabSetId to current tableset value
+	    tabSetId = _pDBMng->getTabSetId(tableSet);
+	}
+
+
+	// register data files and objects
+	
+	if ( pAH ) 
+            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering datafiles ...\n")); 
+	_pGTM->regDataFiles(tableSet);
+
+	if ( pAH ) 
+            pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Registering objects ...\n")); 
+	_pGTM->registerObjects(tableSet);
+
+       
+	unsigned long long cplsn  = _pDBMng->getCommittedLSN(tableSet);
+	_pDBMng->log(_modId, Logger::NOTICE, Chain("Committed lsn = ") + Chain(cplsn));
+	
+	if ( pAH ) 
 		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Committed lsn is ") + Chain(cplsn) + Chain("\n"));
+
+
+	if ( needFileRecovery )
+	{
 	    
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Starting datafile recovery for tableset ") + tableSet + Chain(" ..."));
 	    unsigned long long lsn = dataFileRecovery(tableSet, tabSetId, cplsn, pAH);
 	    
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Datafile recovery for tableset ") + tableSet + Chain(" finished with lsn ") + Chain(lsn));    
-	    tsTicket.close();
-	    
-#ifdef CGDEBUG	
-	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Removing backup ticket ") + tsTicket.getFileName() + Chain(" ..."));
-#endif
-	    
-	    tsTicket.remove();	    
-	}
-	else
-	{	
-	    cplsn = _pDBMng->getCommittedLSN(tableSet);
 
-	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Committed lsn = ") + Chain(cplsn));
-
-	    if ( pAH ) 
-		pAH->syncWithInfo(Chain("local"), Chain("local"), Chain("Committed lsn is ") + Chain(cplsn) + Chain("\n"));
 	}
 	
+
 	if ( pit == 0 )
 	{
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Starting up-to-crash transaction recovery for tableset ") + tableSet + Chain(" ..."));
