@@ -835,7 +835,7 @@ void CegoSelect::checkValidRef()
     {	
 	if ( (*pAD)->isValid() == false )
 	{
-	    throw Exception(EXLOC, Chain("Unknown attribute ") + (*pAD)->toChain() + Chain(" for query ") + toChain(0)); 
+	    throw Exception(EXLOC, Chain("Unknown attribute ") + (*pAD)->toChain()); //  + Chain(" for query ") + toChain(0)); 
 	}
 
 #ifdef CGDEBUG
