@@ -132,6 +132,7 @@ public:
     void setMediator(const Chain& tabSetName, const Chain& hostName);
     void getMedList(const Chain& hostname, ListT<Chain>& medList);
     void getTSforMedAndPrim(const Chain& mediator, const Chain& primary, ListT<Chain>& tsList);
+
     int getTabSetId(const Chain& tabSetName) const;
     Chain getTabSetName(int tabSetId);
     int getTmpFid(const Chain& tabSetName);
@@ -175,7 +176,7 @@ public:
 			const Chain& secondary,
 			const Chain& mediator,
 			int sysFileId, int tempFileId, int sysFileSize, int tmpFileSize, int appFileSize,
-			int logFileSize, int logFileNum, unsigned long long sortAreaSize);
+			int logFileSize, int logFileNum, unsigned long long sortAreaSize, int branchId);
     void rmTableSetDef(const Chain& tabSetName);
     void setDBHost(const Chain& dbHost);
     void getDBHost(Chain& dbHost);
